class Dos2D {

    constructor() {

        let name = arguments[0] || "viewport";

        // Если аргумент -- число, то задается фрейм
        if (typeof name === 'number') {
            this._frame_time = name;
            name = 'viewport';
        } else {
            this._frame_time = 20;
        }

        this._assignconst();
        this._canvas_init(name);
        this._assignevent();
        this.init();
        this._timeout();
    }

    // ПРОТОТИПЫ
    // =================================================================

    init() { }
    loop() { }

    on_mouse_down() { }
    on_mouse_up()   { }
    on_mouse_move() { }
    on_key_down()   { }
    on_key_up()     { }
    on_contextmenu() { }

    // ВНУТРЕННИЕ PRIVATE РУТИНЫ
    // =================================================================

    // Инициализация Canvas
    _canvas_init(id) {

        this._element = document.getElementById(id);
        if (this._element == null) return;

        this._context   = this._element.getContext('2d');
        this._width     = this._element.width;
        this._height    = this._element.height;
        this._img       = this._context.getImageData(0, 0, this._element.width, this._element.height);
        this._factorx   = this._element.width  / this._element.getBoundingClientRect().width;
        this._factory   = this._element.height / this._element.getBoundingClientRect().height;
        this._audioctx  = null;
        this._refresh   = 1;

        this.font(16);
        this.color();
        this.locate(0, 0);
        this.srand = [0, 1];

        // Управление мышью
        this.mouse    = {x: 0, y: 0, btn: 0, press: 0};
        this.keyboard = {key: "", press: 0};
    }

    // Регистрация событий на Canvas
    _assignevent() {

        if (this._element === null)
            return;

        // Работа с мышью
        this._element.addEventListener("mousemove",   function(e) { this._mouse_event(e); this.on_mouse_move(); }.bind(this));
        this._element.addEventListener("mousedown",   function(e) { this._mouse_event(e, e.buttons + 1); this.on_mouse_down(); }.bind(this));
        this._element.addEventListener("mouseup",     function(e) { this._mouse_event(e, 0); this.on_mouse_up(); }.bind(this));
        this._element.addEventListener("contextmenu", function(e) { this.on_contextmenu(e); }.bind(this));

        // Нажатие на клавиши
        window.addEventListener("keydown", function(e) { this._keyboard_event(e); this.on_key_down(); }.bind(this));
        window.addEventListener("keyup",   function(e) { this._keyboard_event(e); this.on_key_up(); }.bind(this));
    }

    // Задание констант: цвета, шрифты
    _assignconst() {

        // Дизеринг паттерном
        this._dither = [
            [ 0, 32,  8, 40,  2, 34, 10, 42],
            [48, 16, 56, 24, 50, 18, 58, 26],
            [12, 44,  4, 36, 14, 46,  6, 38],
            [60, 28, 52, 20, 62, 30, 54, 22],
            [ 3, 35, 11, 43,  1, 33,  9, 41],
            [51, 19, 59, 27, 49, 17, 57, 25],
            [15, 47,  7, 39, 13, 45,  5, 37],
            [63, 31, 55, 23, 61, 29, 53, 21],
        ];

        // Задать dos-цвета
        this._doscolor = [
            0x000000, 0x0000aa, 0x00aa00, 0x00aaaa, 0xaa0000, 0xaa00aa, 0xaa5500, 0xaaaaaa, // 0
            0x555555, 0x5555ff, 0x55ff55, 0x55ffff, 0xff5555, 0xff55ff, 0xffff55, 0xffffff, // 8
            0x000000, 0x141414, 0x202020, 0x2c2c2c, 0x383838, 0x454545, 0x515151, 0x616161, // 10
            0x717171, 0x828282, 0x929292, 0xa2a2a2, 0xb6b6b6, 0xcbcbcb, 0xe3e3e3, 0xffffff, // 18
            0x0000ff, 0x4100ff, 0x7d00ff, 0xbe00ff, 0xff00ff, 0xff00be, 0xff007d, 0xff0041, // 20
            0xff0000, 0xff4100, 0xff7d00, 0xffbe00, 0xffff00, 0xbeff00, 0x7dff00, 0x41ff00, // 28
            0x00ff00, 0x00ff41, 0x00ff7d, 0x00ffbe, 0x00ffff, 0x00beff, 0x007dff, 0x0041ff, // 30
            0x7d7dff, 0x9e7dff, 0xbe7dff, 0xdf7dff, 0xff7dff, 0xff7ddf, 0xff7dbe, 0xff7d9e, // 38
            0xff7d7d, 0xff9e7d, 0xffbe7d, 0xffdf7d, 0xffff7d, 0xdfff7d, 0xbeff7d, 0x9eff7d, // 40
            0x7dff7d, 0x7dff9e, 0x7dffbe, 0x7dffdf, 0x7dffff, 0x7ddfff, 0x7dbeff, 0x7d9eff, // 48
            0xb6b6ff, 0xc7b6ff, 0xdbb6ff, 0xebb6ff, 0xffb6ff, 0xffb6eb, 0xffb6db, 0xffb6c7, // 50
            0xffb6b6, 0xffc7b6, 0xffdbb6, 0xffebb6, 0xffffb6, 0xebffb6, 0xdbffb6, 0xc7ffb6, // 58
            0xb6ffb6, 0xb6ffc7, 0xb6ffdb, 0xb6ffeb, 0xb6ffff, 0xb6ebff, 0xb6dbff, 0xb6c7ff, // 60
            0x000071, 0x1c0071, 0x380071, 0x550071, 0x710071, 0x710055, 0x710038, 0x71001c, // 68
            0x710000, 0x711c00, 0x713800, 0x715500, 0x717100, 0x557100, 0x387100, 0x1c7100, // 70
            0x007100, 0x00711c, 0x007138, 0x007155, 0x007171, 0x005571, 0x003871, 0x001c71, // 78
            0x383871, 0x453871, 0x553871, 0x613871, 0x713871, 0x713861, 0x713855, 0x713845, // 80
            0x713838, 0x714538, 0x715538, 0x716138, 0x717138, 0x617138, 0x557138, 0x457138, // 88
            0x387138, 0x387145, 0x387155, 0x387161, 0x387171, 0x386171, 0x385571, 0x384571, // 90
            0x515171, 0x595171, 0x615171, 0x695171, 0x715171, 0x715169, 0x715161, 0x715159, // 98
            0x715151, 0x715951, 0x716151, 0x716951, 0x717151, 0x697151, 0x617151, 0x597151, // A0
            0x517151, 0x517159, 0x517161, 0x517169, 0x517171, 0x516971, 0x516171, 0x515971, // A8
            0x000041, 0x100041, 0x200041, 0x300041, 0x410041, 0x410030, 0x410020, 0x410010, // B0
            0x410000, 0x411000, 0x412000, 0x413000, 0x414100, 0x304100, 0x204100, 0x104100, // B8
            0x004100, 0x004110, 0x004120, 0x004130, 0x004141, 0x003041, 0x002041, 0x001041, // C0
            0x202041, 0x282041, 0x302041, 0x382041, 0x412041, 0x412038, 0x412030, 0x412028, // C8
            0x412020, 0x412820, 0x413020, 0x413820, 0x414120, 0x384120, 0x304120, 0x284120, // D0
            0x204120, 0x204128, 0x204130, 0x204138, 0x204141, 0x203841, 0x203041, 0x202841, // D8
            0x2c2c41, 0x302c41, 0x342c41, 0x3c2c41, 0x412c41, 0x412c3c, 0x412c34, 0x412c30, // E0
            0x412c2c, 0x41302c, 0x41342c, 0x413c2c, 0x41412c, 0x3c412c, 0x34412c, 0x30412c, // E8
            0x2c412c, 0x2c4130, 0x2c4134, 0x2c413c, 0x2c4141, 0x2c3c41, 0x2c3441, 0x2c3041, // F0
            0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000  // F8
        ];

        // 256 символов 8x16
        this._font16 = [

            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 00
            0x00,0x00,0x7e,0x81,0xa5,0xa5,0xa5,0x81,0x81,0xbd,0x99,0x81,0x7e,0x00,0x00,0x00, // 01
            0x00,0x00,0x7e,0xff,0xdb,0xdb,0xdb,0xff,0xff,0xc3,0xe7,0xff,0x7e,0x00,0x00,0x00, // 02
            0x00,0x00,0x6c,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0x7c,0x38,0x10,0x00,0x00,0x00,0x00, // 03
            0x00,0x00,0x00,0x00,0x10,0x38,0x7c,0xfe,0x7c,0x38,0x10,0x00,0x00,0x00,0x00,0x00, // 04
            0x00,0x00,0x00,0x18,0x3c,0x3c,0xe7,0xe7,0xe7,0x18,0x18,0x3c,0x00,0x00,0x00,0x00, // 05
            0x00,0x00,0x00,0x18,0x3c,0x7e,0xff,0xff,0x7e,0x18,0x18,0x3c,0x00,0x00,0x00,0x00, // 06
            0x00,0x00,0x00,0x00,0x00,0x18,0x3c,0x3c,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 07
            0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xc3,0xc3,0xe7,0xff,0xff,0xff,0xff,0xff,0xff, // 08
            0x00,0x00,0x00,0x00,0x00,0x3c,0x66,0x42,0x42,0x66,0x3c,0x00,0x00,0x00,0x00,0x00, // 09
            0xff,0xff,0xff,0xff,0xc3,0x99,0xbd,0xbd,0x99,0xc3,0xff,0xff,0xff,0xff,0xff,0xff, // 0A
            0x00,0x00,0x00,0x1e,0x0e,0x1a,0x32,0x78,0xcc,0xcc,0xcc,0x78,0x00,0x00,0x00,0x00, // 0B
            0x00,0x00,0x00,0x3c,0x66,0x66,0x66,0x3c,0x18,0x7e,0x18,0x18,0x00,0x00,0x00,0x00, // 0C
            0x00,0x00,0x00,0x3f,0x33,0x3f,0x30,0x30,0x30,0x70,0xf0,0xe0,0x00,0x00,0x00,0x00, // 0D
            0x00,0x00,0x00,0x7f,0x63,0x7f,0x63,0x63,0x63,0x67,0xe7,0xe6,0xc0,0x00,0x00,0x00, // 0E
            0x00,0x00,0x00,0x18,0x18,0xdb,0x3c,0xe7,0x3c,0xdb,0x18,0x18,0x00,0x00,0x00,0x00, // 0F
            0x00,0x00,0x00,0x80,0xc0,0xe0,0xf8,0xfe,0xf8,0xe0,0xc0,0x80,0x00,0x00,0x00,0x00, // 10
            0x00,0x00,0x00,0x02,0x06,0x0e,0x3e,0xfe,0x3e,0x0e,0x06,0x02,0x00,0x00,0x00,0x00, // 11
            0x00,0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,0x18,0x18,0x18,0x7e,0x3c,0x18,0x00, // 12
            0x00,0x00,0x00,0x66,0x66,0x66,0x66,0x66,0x66,0x00,0x66,0x66,0x00,0x00,0x00,0x00, // 13
            0x00,0x00,0x00,0x7f,0xdb,0xdb,0xdb,0x7b,0x1b,0x1b,0x1b,0x1b,0x00,0x00,0x00,0x00, // 14
            0x00,0x00,0x7c,0xc6,0x60,0x38,0x6c,0xc6,0xc6,0x6c,0x38,0x0c,0xc6,0x7c,0x00,0x00, // 15
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0xfe,0xfe,0x00,0x00,0x00,0x00, // 16
            0x00,0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,0x7e,0x3c,0x18,0x7e,0x00,0x00,0x00, // 17
            0x00,0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x00, // 18
            0x00,0x00,0x00,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x7e,0x3c,0x18,0x00, // 19
            0x00,0x00,0x00,0x00,0x00,0x18,0x0c,0xfe,0x0c,0x18,0x00,0x00,0x00,0x00,0x00,0x00, // 1A
            0x00,0x00,0x00,0x00,0x00,0x30,0x60,0xfe,0x60,0x30,0x00,0x00,0x00,0x00,0x00,0x00, // 1B
            0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0xc0,0xc0,0xfe,0x00,0x00,0x00,0x00,0x00,0x00, // 1C
            0x00,0x00,0x00,0x00,0x00,0x28,0x6c,0xfe,0x6c,0x28,0x00,0x00,0x00,0x00,0x00,0x00, // 1D
            0x00,0x00,0x00,0x00,0x10,0x38,0x38,0x7c,0x7c,0xfe,0xfe,0x00,0x00,0x00,0x00,0x00, // 1E
            0x00,0x00,0x00,0xfe,0x7c,0x38,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 1F
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 20
            0x00,0x00,0x18,0x3c,0x3c,0x3c,0x3c,0x18,0x18,0x18,0x00,0x00,0x18,0x00,0x00,0x00, // 21 !
            0x00,0x66,0x66,0x66,0x66,0x66,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 22 "
            0x00,0x00,0x6c,0x6c,0x6c,0xfe,0x6c,0x6c,0x6c,0xfe,0x6c,0x6c,0x6c,0x00,0x00,0x00, // 23 #
            0x18,0x18,0x18,0x7c,0xc6,0xc2,0xc0,0x7c,0x06,0x86,0xc6,0x7c,0x18,0x18,0x18,0x00, // 24 $
            0x00,0x00,0x00,0x00,0x00,0xc2,0xc6,0x0c,0x18,0x30,0x66,0xc6,0x00,0x00,0x00,0x00, // 25 %
            0x00,0x00,0x38,0x6c,0x6c,0x6c,0x38,0x76,0xdc,0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00, // 26 &
            0x00,0x30,0x30,0x30,0x30,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 27 '
            0x00,0x00,0x0c,0x18,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x18,0x0c,0x00,0x00,0x00, // 28 (
            0x00,0x00,0x30,0x18,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x18,0x30,0x00,0x00,0x00, // 29 )
            0x00,0x00,0x00,0x00,0x66,0x66,0x3c,0xff,0x3c,0x66,0x66,0x00,0x00,0x00,0x00,0x00, // 2A *
            0x00,0x00,0x00,0x00,0x18,0x18,0x18,0x7e,0x18,0x18,0x18,0x00,0x00,0x00,0x00,0x00, // 2B +
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x18,0x18,0x30,0x00,0x00, // 2C ,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 2D -
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00, // 2E .
            0x00,0x00,0x00,0x02,0x06,0x0c,0x18,0x30,0x60,0xc0,0x80,0x00,0x00,0x00,0x00,0x00, // 2F /
            0x00,0x00,0x7c,0xc6,0xc6,0xce,0xde,0xf6,0xf6,0xe6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 30 0
            0x00,0x00,0x18,0x18,0x38,0x78,0x18,0x18,0x18,0x18,0x18,0x18,0x7e,0x00,0x00,0x00, // 31 1
            0x00,0x00,0x7c,0xc6,0xc6,0x06,0x06,0x0c,0x18,0x30,0x60,0xc6,0xfe,0x00,0x00,0x00, // 32 2
            0x00,0x00,0x7c,0xc6,0x06,0x06,0x06,0x3c,0x06,0x06,0x06,0xc6,0x7c,0x00,0x00,0x00, // 33 3
            0x00,0x00,0x0c,0x1c,0x3c,0x6c,0xcc,0xcc,0xfe,0x0c,0x0c,0x0c,0x1e,0x00,0x00,0x00, // 34 4
            0x00,0x00,0xfe,0xc0,0xc0,0xc0,0xfc,0x06,0x06,0x06,0x06,0xc6,0x7c,0x00,0x00,0x00, // 35 5
            0x00,0x00,0x38,0x60,0xc0,0xc0,0xfc,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 36 6
            0x00,0x00,0xfe,0xc6,0xc6,0x06,0x06,0x0c,0x18,0x30,0x30,0x30,0x30,0x00,0x00,0x00, // 37 7
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0x7c,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 38 8
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x06,0x0c,0x78,0x00,0x00,0x00, // 39 9
            0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00, // 3A :
            0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x00,0x18,0x18,0x18,0x18,0x30,0x00,0x00, // 3B ;
            0x00,0x00,0x00,0x06,0x0c,0x18,0x30,0x60,0x30,0x18,0x0c,0x06,0x00,0x00,0x00,0x00, // 3C <
            0x00,0x00,0x00,0x00,0x00,0x00,0x7e,0x00,0x00,0x7e,0x00,0x00,0x00,0x00,0x00,0x00, // 3D =
            0x00,0x00,0x00,0x60,0x30,0x18,0x0c,0x06,0x0c,0x18,0x30,0x60,0x00,0x00,0x00,0x00, // 3E >
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0x0c,0x18,0x18,0x18,0x00,0x18,0x18,0x00,0x00,0x00, // 3F ?
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xde,0xde,0xde,0xdc,0xc0,0xc0,0x7c,0x00,0x00,0x00, // 40 @
            0x00,0x00,0x10,0x38,0x6c,0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 41 A
            0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x7c,0x66,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // 42 B
            0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc0,0xc0,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 43 C
            0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // 44 D
            0x00,0x00,0xfe,0x66,0x62,0x60,0x68,0x78,0x68,0x68,0x62,0x66,0xfe,0x00,0x00,0x00, // 45 E
            0x00,0x00,0xfe,0x66,0x62,0x60,0x68,0x78,0x68,0x68,0x60,0x60,0xf0,0x00,0x00,0x00, // 46 F
            0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc0,0xde,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 47 G
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 48 H
            0x00,0x00,0x3c,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 49 I
            0x00,0x00,0x1e,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0xcc,0xcc,0x78,0x00,0x00,0x00, // 4A J
            0x00,0x00,0xe6,0x66,0x66,0x6c,0x6c,0x78,0x6c,0x6c,0x66,0x66,0xe6,0x00,0x00,0x00, // 4B K
            0x00,0x00,0xf0,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x62,0x66,0xfe,0x00,0x00,0x00, // 4C L
            0x00,0x00,0xc6,0xc6,0xee,0xfe,0xfe,0xd6,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 4D M
            0x00,0x00,0xc6,0xc6,0xe6,0xf6,0xfe,0xde,0xce,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 4E N
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 4F O
            0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x66,0x7c,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // 50 P
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xd6,0xde,0x7c,0x0c,0x00,0x00,0x00, // 51 Q
            0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x7c,0x6c,0x66,0x66,0x66,0xe6,0x00,0x00,0x00, // 52 R
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0x60,0x38,0x0c,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 53 S
            0x00,0x00,0x7e,0x7e,0x5a,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 54 T
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 55 U
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x6c,0x38,0x10,0x00,0x00,0x00, // 56 V
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xd6,0xd6,0xfe,0x7c,0x6c,0x6c,0x00,0x00,0x00, // 57 W
            0x00,0x00,0xc6,0xc6,0xc6,0x6c,0x38,0x38,0x38,0x6c,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 58 X
            0x00,0x00,0x66,0x66,0x66,0x66,0x66,0x3c,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 59 Y
            0x00,0x00,0xfe,0xc6,0xc6,0x8c,0x18,0x30,0x60,0xc2,0xc6,0xc6,0xfe,0x00,0x00,0x00, // 5A Z
            0x00,0x00,0x3c,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x3c,0x00,0x00,0x00, // 5B [
            0x00,0x00,0x00,0x80,0xc0,0xe0,0x70,0x38,0x1c,0x0e,0x06,0x02,0x00,0x00,0x00,0x00, // 5C \ symbol
            0x00,0x00,0x3c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x3c,0x00,0x00,0x00, // 5D ]
            0x10,0x10,0x38,0x6c,0xc6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 5E ^
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00, // 5F _
            0x30,0x30,0x30,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 60 `
            0x00,0x00,0x00,0x00,0x00,0x78,0xcc,0x0c,0x7c,0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00, // 61 a
            0x00,0x00,0xe0,0x60,0x60,0x60,0x78,0x6c,0x66,0x66,0x66,0x66,0x7c,0x00,0x00,0x00, // 62 b
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 63 c
            0x00,0x00,0x1c,0x0c,0x0c,0x0c,0x3c,0x6c,0xcc,0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00, // 64 d
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xfe,0xc0,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 65 e
            0x00,0x00,0x38,0x6c,0x64,0x60,0xf0,0x60,0x60,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // 66 f
            0x00,0x00,0x00,0x00,0x00,0x76,0xcc,0xcc,0xcc,0xcc,0xcc,0x7c,0x0c,0xcc,0x78,0x00, // 67 g
            0x00,0x00,0xe0,0x60,0x60,0x6c,0x76,0x66,0x66,0x66,0x66,0x66,0xe6,0x00,0x00,0x00, // 68 h
            0x00,0x00,0x18,0x18,0x00,0x38,0x18,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 69 i
            0x00,0x00,0x06,0x06,0x00,0x0e,0x06,0x06,0x06,0x06,0x06,0x06,0x66,0x66,0x3c,0x00, // 6A j
            0x00,0x00,0xe0,0x60,0x60,0x66,0x66,0x6c,0x78,0x6c,0x66,0x66,0xe6,0x00,0x00,0x00, // 6B k
            0x00,0x00,0x38,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 6C l
            0x00,0x00,0x00,0x00,0x00,0xec,0xfe,0xd6,0xd6,0xd6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 6D m
            0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x00,0x00,0x00, // 6E n
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 6F o
            0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x66,0x66,0x66,0x66,0x66,0x7c,0x60,0xf0,0x00, // 70 p
            0x00,0x00,0x00,0x00,0x00,0x76,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0x7c,0x0c,0x1e,0x00, // 71 q
            0x00,0x00,0x00,0x00,0x00,0xdc,0x76,0x66,0x60,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // 72 r
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0x70,0x1c,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 73 s
            0x00,0x00,0x10,0x30,0x30,0xfc,0x30,0x30,0x30,0x30,0x30,0x36,0x1c,0x00,0x00,0x00, // 74 t
            0x00,0x00,0x00,0x00,0x00,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00, // 75 u
            0x00,0x00,0x00,0x00,0x00,0x66,0x66,0x66,0x66,0x66,0x66,0x3c,0x18,0x00,0x00,0x00, // 76 v
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,0xd6,0xd6,0xfe,0x6c,0x6c,0x00,0x00,0x00, // 77 w
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0x6c,0x38,0x38,0x6c,0xc6,0xc6,0x00,0x00,0x00, // 78 x
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x0c,0xf8,0x00, // 79 y
            0x00,0x00,0x00,0x00,0x00,0xfe,0xc6,0xcc,0x18,0x30,0x66,0xc6,0xfe,0x00,0x00,0x00, // 7A z
            0x00,0x0e,0x18,0x18,0x18,0x18,0x18,0x70,0x70,0x18,0x18,0x18,0x18,0x18,0x0e,0x00, // 7B {
            0x00,0x00,0x18,0x18,0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x18,0x18,0x00,0x00,0x00, // 7C |
            0x00,0x70,0x18,0x18,0x18,0x18,0x18,0x0e,0x0e,0x18,0x18,0x18,0x18,0x18,0x70,0x00, // 7D }
            0x00,0x00,0x76,0xdc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 7E ~
            0x00,0x00,0x00,0x00,0x10,0x38,0x6c,0xc6,0xc6,0xc6,0xc6,0xfe,0x00,0x00,0x00,0x00, // 7F
            0x00,0x00,0x10,0x38,0x6c,0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 80
            0x00,0x00,0xfe,0x66,0x62,0x60,0x7c,0x66,0x66,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // 81
            0x00,0x00,0xfc,0x66,0x66,0x66,0x7c,0x66,0x66,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // 82
            0x00,0x00,0xfe,0x66,0x62,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // 83
            0x00,0x00,0x3e,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xff,0xc3,0xc3,0x00, // 84
            0x00,0x00,0xfe,0x66,0x66,0x62,0x68,0x78,0x68,0x62,0x66,0x66,0xfe,0x00,0x00,0x00, // 85
            0x00,0x00,0xd6,0xd6,0xd6,0x7c,0x38,0x7c,0xd6,0xd6,0xd6,0xd6,0xd6,0x00,0x00,0x00, // 86
            0x00,0x00,0x7c,0xc6,0x06,0x06,0x06,0x3c,0x06,0x06,0x06,0xc6,0x7c,0x00,0x00,0x00, // 87
            0x00,0x00,0xc6,0xc6,0xce,0xde,0xfe,0xf6,0xe6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 88
            0x38,0x38,0xc6,0xc6,0xce,0xde,0xfe,0xf6,0xe6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 89
            0x00,0x00,0xe6,0x66,0x6c,0x6c,0x78,0x6c,0x6c,0x66,0x66,0x66,0xe6,0x00,0x00,0x00, // 8A
            0x00,0x00,0x3e,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xe6,0x00,0x00,0x00, // 8B
            0x00,0x00,0xc6,0xee,0xfe,0xfe,0xd6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 8C
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 8D
            0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 8E
            0x00,0x00,0xfe,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 8F
            0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x66,0x7c,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // 90
            0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc0,0xc0,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // 91
            0x00,0x00,0x7e,0x7e,0x5a,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // 92
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x06,0x06,0xc6,0x7c,0x00,0x00,0x00, // 93
            0x00,0x00,0x18,0x7e,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0x7e,0x18,0x00,0x00,0x00, // 94
            0x00,0x00,0xc6,0xc6,0xc6,0x6c,0x38,0x38,0x38,0x6c,0xc6,0xc6,0xc6,0x00,0x00,0x00, // 95
            0x00,0x00,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xfe,0x06,0x06,0x00, // 96
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x06,0x06,0x06,0x00,0x00,0x00, // 97
            0x00,0x00,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xfe,0x00,0x00,0x00, // 98
            0x00,0x00,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xfe,0x03,0x03,0x00, // 99
            0x00,0x00,0xf8,0xf0,0xb0,0x30,0x3c,0x36,0x36,0x36,0x36,0x36,0x7c,0x00,0x00,0x00, // 9A
            0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xf6,0xde,0xde,0xde,0xde,0xde,0xf6,0x00,0x00,0x00, // 9B
            0x00,0x00,0xf0,0x60,0x60,0x60,0x7c,0x66,0x66,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // 9C
            0x00,0x00,0x78,0xcc,0x86,0x86,0x26,0x3e,0x26,0x86,0x86,0xcc,0x78,0x00,0x00,0x00, // 9D
            0x00,0x00,0x9c,0xb6,0xb6,0xb6,0xb6,0xf6,0xb6,0xb6,0xb6,0xb6,0x9c,0x00,0x00,0x00, // 9E
            0x00,0x00,0x7e,0xcc,0xcc,0xcc,0xcc,0x7c,0x6c,0xcc,0xcc,0xce,0xce,0x00,0x00,0x00, // 9F
            0x00,0x00,0x00,0x00,0x00,0x78,0xcc,0x0c,0x7c,0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00, // A0
            0x00,0x00,0x00,0x1c,0x30,0x60,0x7c,0x66,0x66,0x66,0x66,0x66,0x3c,0x00,0x00,0x00, // A1
            0x00,0x00,0x00,0x00,0x00,0xfc,0x66,0x66,0x7c,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // A2
            0x00,0x00,0x00,0x00,0x00,0xfe,0x62,0x60,0x60,0x60,0x60,0x60,0xf0,0x00,0x00,0x00, // A3
            0x00,0x00,0x00,0x00,0x00,0x3e,0x66,0x66,0x66,0x66,0x66,0x66,0xff,0xc3,0xc3,0x00, // A4
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xfe,0xc0,0xc0,0xc6,0x7c,0x00,0x00,0x00, // A5
            0x00,0x00,0x00,0x00,0x00,0xd6,0xd6,0xd6,0x7c,0x7c,0xd6,0xd6,0xd6,0x00,0x00,0x00, // A6
            0x00,0x00,0x00,0x00,0x00,0x3c,0x66,0x66,0x0c,0x06,0x66,0x66,0x3c,0x00,0x00,0x00, // A7
            0x00,0x00,0x00,0x00,0x00,0xc6,0xce,0xde,0xfe,0xf6,0xe6,0xc6,0xc6,0x00,0x00,0x00, // A8
            0x00,0x00,0x38,0x38,0x00,0xc6,0xce,0xde,0xfe,0xf6,0xe6,0xc6,0xc6,0x00,0x00,0x00, // A9
            0x00,0x00,0x00,0x00,0x00,0xe6,0x66,0x6c,0x78,0x6c,0x66,0x66,0xe6,0x00,0x00,0x00, // AA
            0x00,0x00,0x00,0x00,0x00,0x3e,0x66,0x66,0x66,0x66,0x66,0x66,0xe6,0x00,0x00,0x00, // AB
            0x00,0x00,0x00,0x00,0x00,0xc6,0xee,0xfe,0xfe,0xd6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // AC
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // AD
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00, // AE
            0x00,0x00,0x00,0x00,0x00,0xfe,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00, // AF
            0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11, // B0
            0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55, // B1
            0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd, // B2
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // B3
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xf8,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // B4
            0x18,0x18,0x18,0x18,0x18,0x18,0xf8,0x18,0xf8,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // B5
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xf6,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // B6
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // B7
            0x00,0x00,0x00,0x00,0x00,0x00,0xf8,0x18,0xf8,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // B8
            0x36,0x36,0x36,0x36,0x36,0x36,0xf6,0x06,0xf6,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // B9
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // BA
            0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0x06,0xf6,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // BB
            0x36,0x36,0x36,0x36,0x36,0x36,0xf6,0x06,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // BC
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // BD
            0x18,0x18,0x18,0x18,0x18,0x18,0xf8,0x18,0xf8,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // BE
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf8,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // BF
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // C0
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // C1
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // C2
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x1f,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // C3
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // C4
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xff,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // C5
            0x18,0x18,0x18,0x18,0x18,0x18,0x1f,0x18,0x1f,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // C6
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x37,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // C7
            0x36,0x36,0x36,0x36,0x36,0x36,0x37,0x30,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // C8
            0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x30,0x37,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // C9
            0x36,0x36,0x36,0x36,0x36,0x36,0xf7,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // CA
            0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xf7,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // CB
            0x36,0x36,0x36,0x36,0x36,0x36,0x37,0x30,0x37,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // CC
            0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // CD
            0x36,0x36,0x36,0x36,0x36,0x36,0xf7,0x00,0xf7,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // CE
            0x18,0x18,0x18,0x18,0x18,0x18,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // CF
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // D0
            0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // D1
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // D2
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // D3
            0x18,0x18,0x18,0x18,0x18,0x18,0x1f,0x18,0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // D4
            0x00,0x00,0x00,0x00,0x00,0x00,0x1f,0x18,0x1f,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // D5
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // D6
            0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xff,0x36,0x36,0x36,0x36,0x36,0x36,0x36, // D7
            0x18,0x18,0x18,0x18,0x18,0x18,0xff,0x18,0xff,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // D8
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xf8,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // D9
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // DA
            0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff, // DB
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff, // DC
            0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0, // DD
            0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f, // DE
            0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // DF
            0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x66,0x66,0x66,0x66,0x7c,0x60,0x60,0xf0,0x00, // E0
            0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc6,0xc6,0x7c,0x00,0x00,0x00, // E1
            0x00,0x00,0x00,0x00,0x00,0x7e,0x5a,0x18,0x18,0x18,0x18,0x18,0x3c,0x00,0x00,0x00, // E2
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x0c,0xf8,0x00, // E3
            0x00,0x00,0x00,0x00,0x00,0x18,0x7e,0xdb,0xdb,0xdb,0xdb,0xdb,0x7e,0x18,0x18,0x00, // E4
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0x6c,0x38,0x38,0x6c,0xc6,0xc6,0x00,0x00,0x00, // E5
            0x00,0x00,0x00,0x00,0x00,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xfe,0x06,0x06,0x00, // E6
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,0xc6,0x7e,0x06,0x06,0x06,0x00,0x00,0x00, // E7
            0x00,0x00,0x00,0x00,0x00,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xfe,0x00,0x00,0x00, // E8
            0x00,0x00,0x00,0x00,0x00,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xfe,0x03,0x03,0x00, // E9
            0x00,0x00,0x00,0x00,0x00,0xf8,0xb0,0x3c,0x36,0x36,0x36,0x36,0x7c,0x00,0x00,0x00, // EA
            0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xf6,0xde,0xde,0xde,0xde,0xf6,0x00,0x00,0x00, // EB
            0x00,0x00,0x00,0x00,0x00,0xf0,0x60,0x60,0x7c,0x66,0x66,0x66,0xfc,0x00,0x00,0x00, // EC
            0x00,0x00,0x00,0x00,0x00,0x3c,0x66,0x06,0x1e,0x06,0x66,0x66,0x3c,0x00,0x00,0x00, // ED
            0x00,0x00,0x00,0x00,0x00,0x9c,0xb6,0xb6,0xf6,0xb6,0xb6,0xb6,0x9c,0x00,0x00,0x00, // EE
            0x00,0x00,0x00,0x00,0x00,0x7e,0xcc,0xcc,0xcc,0x7c,0x6c,0xcc,0xce,0x00,0x00,0x00, // EF
            0x00,0x00,0x00,0x00,0xfe,0x00,0x00,0xfe,0x00,0x00,0xfe,0x00,0x00,0x00,0x00,0x00, // F0
            0x00,0x00,0x00,0x00,0x18,0x18,0x7e,0x18,0x18,0x00,0x00,0xff,0x00,0x00,0x00,0x00, // F1
            0x00,0x00,0x00,0x30,0x18,0x0c,0x06,0x0c,0x18,0x30,0x00,0x7e,0x00,0x00,0x00,0x00, // F2
            0x00,0x00,0x00,0x0c,0x18,0x30,0x60,0x30,0x18,0x0c,0x00,0x7e,0x00,0x00,0x00,0x00, // F3
            0x00,0x00,0x00,0x0e,0x1b,0x1b,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18, // F4
            0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xd8,0xd8,0x70,0x00,0x00,0x00,0x00, // F5
            0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x7e,0x00,0x18,0x18,0x00,0x00,0x00,0x00,0x00, // F6
            0x00,0x00,0x00,0x00,0x00,0x76,0xdc,0x00,0x76,0xdc,0x00,0x00,0x00,0x00,0x00,0x00, // F7
            0x00,0x38,0x6c,0x6c,0x6c,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // F8
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // F9
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // FA
            0x00,0x0f,0x0c,0x0c,0x0c,0x0c,0x0c,0xec,0x6c,0x3c,0x1c,0x0c,0x00,0x00,0x00,0x00, // FB
            0x00,0x00,0xd8,0x6c,0x6c,0x6c,0x6c,0x6c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // FC
            0x00,0x00,0x70,0xd8,0x30,0x60,0xc8,0xf8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // FD
            0x00,0x00,0x00,0x00,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x00,0x00,0x00,0x00, // FE
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00  // FF
        ];

        // 96 символов
        this._font8 = [
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x7e, 0x81, 0xa5, 0x81, 0xbd, 0x99, 0x81, 0x7e,
            0x3c, 0x7e, 0xdb, 0xff, 0xc3, 0x7e, 0x3c, 0x00,
            0x00, 0xee, 0xfe, 0xfe, 0x7c, 0x38, 0x10, 0x00,
            0x10, 0x38, 0x7c, 0xfe, 0x7c, 0x38, 0x10, 0x00,
            0x00, 0x3c, 0x18, 0xff, 0xff, 0x08, 0x18, 0x00,
            0x10, 0x38, 0x7c, 0xfe, 0xfe, 0x10, 0x38, 0x00,
            0x00, 0x00, 0x18, 0x3c, 0x18, 0x00, 0x00, 0x00,
            0xff, 0xff, 0xe7, 0xc3, 0xe7, 0xff, 0xff, 0xff,
            0x00, 0x3c, 0x42, 0x81, 0x81, 0x42, 0x3c, 0x00,
            0xff, 0xc3, 0xbd, 0x7e, 0x7e, 0xbd, 0xc3, 0xff,
            0x1f, 0x07, 0x0d, 0x7c, 0xc6, 0xc6, 0x7c, 0x00,
            0x00, 0x7e, 0xc3, 0xc3, 0x7e, 0x18, 0x7e, 0x18,
            0x04, 0x06, 0x07, 0x04, 0x04, 0xfc, 0xf8, 0x00,
            0x0c, 0x0a, 0x0d, 0x0b, 0xf9, 0xf9, 0x1f, 0x1f,
            0x00, 0x92, 0x7c, 0x44, 0xc6, 0x7c, 0x92, 0x00,
            0x00, 0x00, 0x60, 0x78, 0x7e, 0x78, 0x60, 0x00,
            0x00, 0x00, 0x06, 0x1e, 0x7e, 0x1e, 0x06, 0x00,
            0x18, 0x7e, 0x18, 0x18, 0x18, 0x18, 0x7e, 0x18,
            0x66, 0x66, 0x66, 0x66, 0x66, 0x00, 0x66, 0x00,
            0xff, 0xb6, 0x76, 0x36, 0x36, 0x36, 0x36, 0x00,
            0x7e, 0xc1, 0xdc, 0x22, 0x22, 0x1f, 0x83, 0x7e,
            0x00, 0x00, 0x00, 0x7e, 0x7e, 0x00, 0x00, 0x00,
            0x18, 0x7e, 0x18, 0x18, 0x7e, 0x18, 0x00, 0xff,
            0x18, 0x7e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00,
            0x18, 0x18, 0x18, 0x18, 0x18, 0x7e, 0x18, 0x00,
            0x00, 0x04, 0x06, 0xff, 0x06, 0x04, 0x00, 0x00,
            0x00, 0x20, 0x60, 0xff, 0x60, 0x20, 0x00, 0x00,
            0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xff, 0x00,
            0x00, 0x24, 0x66, 0xff, 0x66, 0x24, 0x00, 0x00,
            0x00, 0x00, 0x10, 0x38, 0x7c, 0xfe, 0x00, 0x00,
            0x00, 0x00, 0x00, 0xfe, 0x7c, 0x38, 0x10, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x30, 0x30, 0x30, 0x30, 0x30, 0x00, 0x30, 0x00,
            0x66, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x6c, 0x6c, 0xfe, 0x6c, 0xfe, 0x6c, 0x6c, 0x00,
            0x10, 0x7c, 0xd2, 0x7c, 0x86, 0x7c, 0x10, 0x00,
            0xf0, 0x96, 0xfc, 0x18, 0x3e, 0x72, 0xde, 0x00,
            0x30, 0x48, 0x30, 0x78, 0xce, 0xcc, 0x78, 0x00,
            0x0c, 0x0c, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x10, 0x60, 0xc0, 0xc0, 0xc0, 0x60, 0x10, 0x00,
            0x10, 0x0c, 0x06, 0x06, 0x06, 0x0c, 0x10, 0x00,
            0x00, 0x54, 0x38, 0xfe, 0x38, 0x54, 0x00, 0x00,
            0x00, 0x18, 0x18, 0x7e, 0x18, 0x18, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x70,
            0x00, 0x00, 0x00, 0x7e, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x00,
            0x02, 0x06, 0x0c, 0x18, 0x30, 0x60, 0xc0, 0x00,
            0x7c, 0xce, 0xde, 0xf6, 0xe6, 0xe6, 0x7c, 0x00,
            0x18, 0x38, 0x78, 0x18, 0x18, 0x18, 0x3c, 0x00,
            0x7c, 0xc6, 0x06, 0x0c, 0x30, 0x60, 0xfe, 0x00,
            0x7c, 0xc6, 0x06, 0x3c, 0x06, 0xc6, 0x7c, 0x00,
            0x0e, 0x1e, 0x36, 0x66, 0xfe, 0x06, 0x06, 0x00,
            0xfe, 0xc0, 0xc0, 0xfc, 0x06, 0x06, 0xfc, 0x00,
            0x7c, 0xc6, 0xc0, 0xfc, 0xc6, 0xc6, 0x7c, 0x00,
            0xfe, 0x06, 0x0c, 0x18, 0x30, 0x60, 0x60, 0x00,
            0x7c, 0xc6, 0xc6, 0x7c, 0xc6, 0xc6, 0x7c, 0x00,
            0x7c, 0xc6, 0xc6, 0x7e, 0x06, 0xc6, 0x7c, 0x00,
            0x00, 0x30, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00,
            0x00, 0x30, 0x00, 0x00, 0x00, 0x30, 0x20, 0x00,
            0x00, 0x1c, 0x30, 0x60, 0x30, 0x1c, 0x00, 0x00,
            0x00, 0x00, 0x7e, 0x00, 0x7e, 0x00, 0x00, 0x00,
            0x00, 0x70, 0x18, 0x0c, 0x18, 0x70, 0x00, 0x00,
            0x7c, 0xc6, 0x0c, 0x18, 0x30, 0x00, 0x30, 0x00,
            0x7c, 0x82, 0x9a, 0xaa, 0xaa, 0x9e, 0x7c, 0x00,
            0x7c, 0xc6, 0xc6, 0xfe, 0xc6, 0xc6, 0xc6, 0x00,
            0xfc, 0x66, 0x66, 0x7c, 0x66, 0x66, 0xfc, 0x00,
            0x7c, 0xc6, 0xc0, 0xc0, 0xc0, 0xc6, 0x7c, 0x00,
            0xfc, 0x66, 0x66, 0x66, 0x66, 0x66, 0xfc, 0x00,
            0xfe, 0x62, 0x68, 0x78, 0x68, 0x62, 0xfe, 0x00,
            0xfe, 0x62, 0x68, 0x78, 0x68, 0x60, 0xf0, 0x00,
            0x7c, 0xc6, 0xc6, 0xc0, 0xde, 0xc6, 0x7c, 0x00,
            0xc6, 0xc6, 0xc6, 0xfe, 0xc6, 0xc6, 0xc6, 0x00,
            0x3c, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00,
            0x1e, 0x0c, 0x0c, 0x0c, 0x0c, 0xcc, 0x78, 0x00,
            0xc6, 0xcc, 0xd8, 0xf0, 0xd8, 0xcc, 0xc6, 0x00,
            0xf0, 0x60, 0x60, 0x60, 0x60, 0x62, 0xfe, 0x00,
            0xc6, 0xee, 0xfe, 0xd6, 0xc6, 0xc6, 0xc6, 0x00,
            0xc6, 0xe6, 0xf6, 0xde, 0xce, 0xc6, 0xc6, 0x00,
            0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00,
            0xfc, 0x66, 0x66, 0x7c, 0x60, 0x60, 0xf0, 0x00,
            0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x0c,
            0xfc, 0x66, 0x66, 0x7c, 0x66, 0x66, 0xe6, 0x00,
            0x7c, 0xc6, 0xc0, 0x7c, 0x06, 0xc6, 0x7c, 0x00,
            0x7e, 0x5a, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00,
            0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00,
            0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x6c, 0x38, 0x00,
            0xc6, 0xc6, 0xc6, 0xc6, 0xd6, 0xee, 0xc6, 0x00,
            0xc6, 0x6c, 0x38, 0x38, 0x38, 0x6c, 0xc6, 0x00,
            0x66, 0x66, 0x66, 0x3c, 0x18, 0x18, 0x3c, 0x00,
            0xfe, 0xc6, 0x0c, 0x18, 0x30, 0x66, 0xfe, 0x00,
            0x1c, 0x18, 0x18, 0x18, 0x18, 0x18, 0x1c, 0x00,
            0xc0, 0x60, 0x30, 0x18, 0x0c, 0x06, 0x02, 0x00,
            0x70, 0x30, 0x30, 0x30, 0x30, 0x30, 0x70, 0x00,
            0x00, 0x00, 0x10, 0x38, 0x6c, 0xc6, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff,
            0x30, 0x30, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x7c, 0x06, 0x7e, 0xc6, 0x7e, 0x00,
            0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xc6, 0xfc, 0x00,
            0x00, 0x00, 0x7c, 0xc6, 0xc0, 0xc6, 0x7c, 0x00,
            0x06, 0x06, 0x7e, 0xc6, 0xc6, 0xc6, 0x7e, 0x00,
            0x00, 0x00, 0x7c, 0xc6, 0xfe, 0xc0, 0x7c, 0x00,
            0x3c, 0x66, 0x60, 0xf0, 0x60, 0x60, 0x60, 0x00,
            0x00, 0x00, 0x7e, 0xc6, 0xc6, 0x7e, 0x06, 0x7c,
            0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xc6, 0xc6, 0x00,
            0x18, 0x00, 0x38, 0x18, 0x18, 0x18, 0x3c, 0x00,
            0x00, 0x0c, 0x00, 0x1c, 0x0c, 0x0c, 0xcc, 0x78,
            0xc0, 0xc0, 0xc6, 0xd8, 0xf0, 0xd8, 0xc6, 0x00,
            0x38, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00,
            0x00, 0x00, 0xee, 0xfe, 0xd6, 0xc6, 0xc6, 0x00,
            0x00, 0x00, 0xfc, 0xc6, 0xc6, 0xc6, 0xc6, 0x00,
            0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0x7c, 0x00,
            0x00, 0x00, 0xfc, 0xc6, 0xc6, 0xfc, 0xc0, 0xc0,
            0x00, 0x00, 0x7e, 0xc6, 0xc6, 0x7e, 0x06, 0x06,
            0x00, 0x00, 0xde, 0x76, 0x60, 0x60, 0x60, 0x00,
            0x00, 0x00, 0x7c, 0xc0, 0x7c, 0x06, 0x7c, 0x00,
            0x18, 0x18, 0x7e, 0x18, 0x18, 0x18, 0x1e, 0x00,
            0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0x7e, 0x00,
            0x00, 0x00, 0xc6, 0xc6, 0xc6, 0x6c, 0x38, 0x00,
            0x00, 0x00, 0xc6, 0xc6, 0xd6, 0xfe, 0xc6, 0x00,
            0x00, 0x00, 0xc6, 0x6c, 0x38, 0x6c, 0xc6, 0x00,
            0x00, 0x00, 0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0x7c,
            0x00, 0x00, 0xfe, 0x0c, 0x18, 0x60, 0xfe, 0x00,
            0x0e, 0x18, 0x18, 0x70, 0x18, 0x18, 0x0e, 0x00,
            0x18, 0x18, 0x18, 0x00, 0x18, 0x18, 0x18, 0x00,
            0xe0, 0x30, 0x30, 0x1c, 0x30, 0x30, 0xe0, 0x00,
            0x00, 0x00, 0x70, 0x9a, 0x0e, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x18, 0x3c, 0x66, 0xff, 0x00, 0x00,
            0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88,
            0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa,
            0xdb, 0x77, 0xdb, 0xee, 0xdb, 0x77, 0xdb, 0xee,
            0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x10, 0x10, 0xf0, 0x10, 0x10, 0x10,
            0x10, 0x10, 0xf0, 0x10, 0xf0, 0x10, 0x10, 0x10,
            0x14, 0x14, 0x14, 0x14, 0xf4, 0x14, 0x14, 0x14,
            0x00, 0x00, 0x00, 0x00, 0xfc, 0x14, 0x14, 0x14,
            0x00, 0x00, 0xf0, 0x10, 0xf0, 0x10, 0x10, 0x10,
            0x14, 0x14, 0xf4, 0x04, 0xf4, 0x14, 0x14, 0x14,
            0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14,
            0x00, 0x00, 0xfc, 0x04, 0xf4, 0x14, 0x14, 0x14,
            0x14, 0x14, 0xf4, 0x04, 0xfc, 0x00, 0x00, 0x00,
            0x14, 0x14, 0x14, 0x14, 0xfc, 0x00, 0x00, 0x00,
            0x10, 0x10, 0xf0, 0x10, 0xf0, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0xf0, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x10, 0x10, 0x1f, 0x00, 0x00, 0x00,
            0x10, 0x10, 0x10, 0x10, 0xff, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0xff, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x10, 0x10, 0x1f, 0x10, 0x10, 0x10,
            0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00,
            0x10, 0x10, 0x10, 0x10, 0xff, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x1f, 0x10, 0x1f, 0x10, 0x10, 0x10,
            0x14, 0x14, 0x14, 0x14, 0x17, 0x14, 0x14, 0x14,
            0x14, 0x14, 0x17, 0x10, 0x1f, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x1f, 0x10, 0x17, 0x14, 0x14, 0x14,
            0x14, 0x14, 0xf7, 0x00, 0xff, 0x00, 0x00, 0x00,
            0x00, 0x00, 0xff, 0x00, 0xf7, 0x14, 0x14, 0x14,
            0x14, 0x14, 0x17, 0x10, 0x17, 0x14, 0x14, 0x14,
            0x00, 0x00, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00,
            0x14, 0x14, 0xf7, 0x00, 0xf7, 0x14, 0x14, 0x14,
            0x10, 0x10, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00,
            0x14, 0x14, 0x14, 0x14, 0xff, 0x00, 0x00, 0x00,
            0x00, 0x00, 0xff, 0x00, 0xff, 0x10, 0x10, 0x10,
            0x00, 0x00, 0x00, 0x00, 0xff, 0x14, 0x14, 0x14,
            0x14, 0x14, 0x14, 0x14, 0x1f, 0x00, 0x00, 0x00,
            0x10, 0x10, 0x1f, 0x10, 0x1f, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x1f, 0x10, 0x1f, 0x10, 0x10, 0x10,
            0x00, 0x00, 0x00, 0x00, 0x1f, 0x14, 0x14, 0x14,
            0x14, 0x14, 0x14, 0x14, 0xff, 0x14, 0x14, 0x14,
            0x10, 0x10, 0xff, 0x10, 0xff, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x10, 0x10, 0xf0, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x1f, 0x10, 0x10, 0x10,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
            0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0,
            0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
            0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00,
            0x10, 0x10, 0x7c, 0x10, 0x10, 0x00, 0x7c, 0x00,
            0x60, 0x30, 0x18, 0x30, 0x60, 0x00, 0x78, 0x00,
            0x18, 0x30, 0x60, 0x30, 0x18, 0x00, 0x78, 0x00,
            0x08, 0x14, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10,
            0x10, 0x10, 0x10, 0x10, 0x10, 0x50, 0x20, 0x00,
            0x30, 0x30, 0x00, 0xfc, 0x00, 0x30, 0x30, 0x00,
            0x00, 0x00, 0x64, 0x98, 0x00, 0x64, 0x98, 0x00,
            0x60, 0x90, 0x90, 0x60, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x30, 0x30, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00,
            0x1f, 0x10, 0x10, 0x90, 0x50, 0x30, 0x10, 0x00,
            0xa0, 0xd0, 0x90, 0x90, 0x90, 0x00, 0x00, 0x00,
            0x60, 0x90, 0x20, 0x40, 0xf0, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x3c, 0x3c, 0x3c, 0x3c, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x0e, 0x1e, 0x36, 0x66, 0xfe, 0xc6, 0xc6, 0x00,
            0xfc, 0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xfc, 0x00,
            0xf8, 0xcc, 0xcc, 0xfc, 0xc6, 0xc6, 0xfc, 0x00,
            0xfc, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
            0x7e, 0x66, 0x66, 0x66, 0x66, 0x66, 0xff, 0xc3,
            0xfc, 0xc0, 0xc0, 0xf8, 0xc0, 0xc0, 0xfe, 0x00,
            0xdb, 0xdb, 0x7e, 0x18, 0x7e, 0xdb, 0xdb, 0x00,
            0x3c, 0x66, 0x06, 0x3c, 0x06, 0xc6, 0x7c, 0x00,
            0xc6, 0xc6, 0xce, 0xde, 0xf6, 0xe6, 0xc6, 0x00,
            0xd6, 0xc6, 0xce, 0xde, 0xf6, 0xe6, 0xc6, 0x00,
            0xc6, 0xcc, 0xd8, 0xf8, 0xcc, 0xc6, 0xc6, 0x00,
            0x06, 0x0e, 0x1e, 0x36, 0x66, 0xc6, 0xc6, 0x00,
            0xc6, 0xee, 0xfe, 0xd6, 0xc6, 0xc6, 0xc6, 0x00,
            0xc6, 0xc6, 0xc6, 0xfe, 0xc6, 0xc6, 0xc6, 0x00,
            0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00,
            0xfe, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x00,
            0xfc, 0xc6, 0xc6, 0xfc, 0xc0, 0xc0, 0xc0, 0x00,
            0x7c, 0xc6, 0xc0, 0xc0, 0xc0, 0xc6, 0x7c, 0x00,
            0xfc, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x00,
            0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0xc6, 0x7c, 0x00,
            0x18, 0x7e, 0xdb, 0xdb, 0xdb, 0x7e, 0x18, 0x00,
            0xc3, 0x66, 0x3c, 0x18, 0x3c, 0x66, 0xc3, 0x00,
            0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xfe, 0x06,
            0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0x06, 0x06, 0x00,
            0xd6, 0xd6, 0xd6, 0xd6, 0xd6, 0xd6, 0xfe, 0x00,
            0xd6, 0xd6, 0xd6, 0xd6, 0xd6, 0xd6, 0xff, 0x03,
            0xf0, 0x30, 0x30, 0x3e, 0x33, 0x33, 0x3e, 0x00,
            0xc2, 0xc2, 0xc2, 0xf2, 0xda, 0xda, 0xf2, 0x00,
            0xc0, 0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xfc, 0x00,
            0x7c, 0xc6, 0x06, 0x1e, 0x06, 0xc6, 0x7c, 0x00,
            0xce, 0xdb, 0xdb, 0xfb, 0xdb, 0xdb, 0xce, 0x00,
            0x7e, 0xc6, 0xc6, 0x7e, 0x36, 0x66, 0xc6, 0x00,
            0x00, 0x00, 0x78, 0x0c, 0x7c, 0xcc, 0x7e, 0x00,
            0x04, 0x78, 0xc0, 0xf8, 0xcc, 0xcc, 0x78, 0x00,
            0x00, 0x00, 0xf8, 0xcc, 0xf8, 0xc6, 0xfc, 0x00,
            0x00, 0x00, 0xfc, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
            0x00, 0x00, 0x7e, 0x66, 0x66, 0x66, 0xff, 0xc3,
            0x00, 0x00, 0x78, 0xcc, 0xfc, 0xc0, 0x7c, 0x00,
            0x00, 0x00, 0xdb, 0x7e, 0x18, 0x7e, 0xdb, 0x00,
            0x00, 0x00, 0x78, 0xcc, 0x18, 0xcc, 0x78, 0x00,
            0x00, 0x00, 0xcc, 0xcc, 0xdc, 0xec, 0xcc, 0x00,
            0x30, 0x00, 0xcc, 0xcc, 0xdc, 0xec, 0xcc, 0x00,
            0x00, 0x00, 0xcc, 0xd8, 0xf0, 0xcc, 0xcc, 0x00,
            0x00, 0x00, 0x0e, 0x1e, 0x36, 0x66, 0xc6, 0x00,
            0x00, 0x00, 0xc6, 0xee, 0xd6, 0xc6, 0xc6, 0x00,
            0x00, 0x00, 0xcc, 0xcc, 0xfc, 0xcc, 0xcc, 0x00,
            0x00, 0x00, 0x78, 0xcc, 0xcc, 0xcc, 0x78, 0x00,
            0x00, 0x00, 0xfc, 0xcc, 0xcc, 0xcc, 0xcc, 0x00,
            0x00, 0x00, 0xf8, 0xcc, 0xcc, 0xf8, 0xc0, 0xc0,
            0x00, 0x00, 0x78, 0xcc, 0xc0, 0xcc, 0x78, 0x00,
            0x00, 0x00, 0xfc, 0x30, 0x30, 0x30, 0x30, 0x00,
            0x00, 0x00, 0xcc, 0xcc, 0x7c, 0x0c, 0xcc, 0x78,
            0x00, 0x18, 0x7e, 0xdb, 0xdb, 0x7e, 0x18, 0x18,
            0x00, 0x00, 0xc6, 0x6c, 0x38, 0x6c, 0xc6, 0x00,
            0x00, 0x00, 0xcc, 0xcc, 0xcc, 0xcc, 0xfe, 0x06,
            0x00, 0x00, 0xcc, 0xcc, 0x7c, 0x0c, 0x0c, 0x00,
            0x00, 0x00, 0xd6, 0xd6, 0xd6, 0xd6, 0xfe, 0x00,
            0x00, 0x00, 0xd6, 0xd6, 0xd6, 0xd6, 0xff, 0x03,
            0x00, 0x00, 0xf0, 0x30, 0x3e, 0x33, 0x3e, 0x00,
            0x00, 0x00, 0xc2, 0xc2, 0xf2, 0xda, 0xf2, 0x00,
            0x00, 0x00, 0xc0, 0xc0, 0xf8, 0xcc, 0xf8, 0x00,
            0x00, 0x00, 0x7c, 0xc6, 0x1e, 0xc6, 0x7c, 0x00,
            0x00, 0x00, 0xce, 0xdb, 0xfb, 0xdb, 0xce, 0x00,
            0x00, 0x00, 0x7c, 0xcc, 0x7c, 0x6c, 0xcc, 0x00,
        ];
    }

    // Нарисовать данные из буфера
    _flush() {
        this._context.putImageData(this._img, 0, 0);
    }

    // Обновляет картинку при любых изменениях
    _timeout() {

        if (this._refresh) this._flush();
        this._refresh = 0;
        this.loop();
        setTimeout(function() { this._timeout(); }.bind(this), this._frame_time);
    }

    // Обработка событий мыши
    _mouse_event(e, p = null) {

        this._event = e;
        if (p !== null) this.mouse.press = p;

        e.stopPropagation();
        e.preventDefault();

        let x = Math.trunc(e.offsetX * this._factorx);
        let y = Math.trunc(e.offsetY * this._factory);

        [this.mouse.x, this.mouse.y, this.mouse.btn] = [x, y, e.buttons];
    }

    // Обработка событий клавиатуры
    _keyboard_event(e) {

        this._event = e;
        this.keyboard.key = e.key;
    }

    // PROTECTED (Работа с текстом)
    // =================================================================

    // Печать в режиме телетайпа
    _print_char(ch) {

        // Это не перенос символа, печатать
        if (ch === 10) {
            this._print_nl();

        } else {

            // Напечатать символ
            this.print_font(this._locate_x, this._locate_y, ch);

            // Передвинуть вперед
            this._locate_x += 8;

            // Перенос строки
            if (this._locate_x >= this._width)
                this._print_nl();
        }
    }

    // Перенос строки
    _print_nl() {

        this._locate_x  = 0;
        this._locate_y += this._font_height;

        // Превышение высоты
        if (this._locate_y >= this._height) {
            this._locate_y -= this._font_height;
            this._print_scroll_up();
        }
    }

    // Отскроллить страницу вверх
    _print_scroll_up(h = 0) {

        h = h ? h : this._font_height;

        // Сдвиг
        for (let y = 0; y < this._height - h; y++)
        for (let x = 0; x < this._width; x++)
            this.pset(x, y, this.point(x, y+h));

        // Очистка области
        for (let y = 0; y < h; y++)
        for (let x = 0; x < this._width; x++)
            this.pset(x, y + this._height - h, this._back ? this._back : 0);
    }

    // РИСОВАНИЕ НА CANVAS
    // =================================================================

    // Загрузка изображения в документ
    load_image(src) {

        let img = new Image();
        let result = {
            ok:      false,
            element: null,
            width:   0,
            height:  0,
            context: null,
            img:     null,
            point: function(x, y) {

                if (this.ok) {

                    if (x < 0 || y < 0) {
                        return 0;
                    }

                    [x, y] = [Math.trunc(x) % this.width, Math.trunc(y) % this.height];

                    let p = 4*(x + y*this.width);
                    return ((this.img.data[p + 3] ^ 0xff) << 24) +
                            (this.img.data[p] << 16) +
                            (this.img.data[p + 1] << 8) +
                            (this.img.data[p + 2]);
                }

                return 0;
            }
        };

        img.src = src;
        img.onload = function() {

            let canvas = document.createElement('canvas');

            canvas.style.display = 'none';
            canvas.width  = img.width;
            canvas.height = img.height;

            document.body.appendChild(canvas);

            result.element = canvas;
            result.context = canvas.getContext('2d');
            result.context.drawImage(img, 0, 0, canvas.width, canvas.height);
            result.img     = result.context.getImageData(0, 0, canvas.width, canvas.height);
            result.width   = canvas.width;
            result.height  = canvas.height;
            result.ok      = true;

            img = null;

        }.bind(this);

        return result;
    }

    // Получение DOS-цвета
    dos(id) {
        return this._doscolor[ Math.floor(id) & 255 ];
    }

    // Очистка экрана
    cls(cl = 0) {

        for (let y = 0; y < this._height; y++)
        for (let x = 0; x < this._width; x++) {
            this.pset(x, y, cl);
        }

        this.locate(0, 0);
    }

    // Копирование из буфера
    copy(buf) {

        let n = 0, p = 0;
        for (let y = 0; y < this._height; y++)
        for (let x = 0; x < this._width;  x++) {

            let c = buf[n++];
            this._img.data[p++] =  (c >> 16) & 0xff;
            this._img.data[p++] =  (c >>  8) & 0xff;
            this._img.data[p++] =  (c      ) & 0xff;
            this._img.data[p++] = ((c >> 24) & 0xff) ^ 0xff;
        }

        this._refresh = 1;
    }

    // Рисование пикселя
    pset(x, y, c) {

        x = Math.floor(x);
        y = Math.floor(y);

        this._refresh = 1;

        if (x >= 0 && y >= 0 && x < this._width && y < this._height) {

            let p = 4*(x + y*this._width);

            this._img.data[p    ] =  (c >> 16) & 0xff;
            this._img.data[p + 1] =  (c >>  8) & 0xff;
            this._img.data[p + 2] =  (c      ) & 0xff;
            this._img.data[p + 3] = ((c >> 24) & 0xff) ^ 0xff;
        }
    }

    // Цвет точки
    point(x, y) {

        x = Math.trunc(x);
        y = Math.trunc(y);

        if (x < 0 || y < 0 || x > this._width || y > this._height) {
            return 0;
        }

        let p = 4*(x + y*this._width);

        let r = this._img.data[p    ];
        let g = this._img.data[p + 1];
        let b = this._img.data[p + 2];
        let a = this._img.data[p + 3] ^ 0xff;

        return (a<<24) + (r<<16) + (g<<8) + b;
    }

    // Рисование блока
    block(x1, y1, x2, y2, cl) {

        for (let y = y1; y <= y2; y++)
        for (let x = x1; x <= x2; x++)
            this.pset(x, y, cl);
    }

    // Нарисовать линию на экране
    line(x1, y1, x2, y2, c) {

        [x1, y1] = [Math.floor(x1), Math.floor(y1)];
        [x2, y2] = [Math.floor(x2), Math.floor(y2)];

        let deltax = Math.abs(x2 - x1);
        let deltay = Math.abs(y2 - y1);
        let signx  = x1 < x2 ? 1 : -1;
        let signy  = y1 < y2 ? 1 : -1;

        let error = deltax - deltay;
        let error2;

        this.pset(x2, y2, c);

        while (x1 !== x2 || y1 !== y2)
        {
            this.pset(x1, y1, c);
            error2 = error * 2;

            if (error2 > -deltay) {
                error -= deltay;
                x1 += signx;
            }

            if (error2 < deltax) {
                error += deltax;
                y1 += signy;
            }
        }
    }

    // Рисовать незакрашенную или закрашенную окружность
    circle(xc, yc, radius, color, fill = 0) {

        let x = 0,
            y = radius,
            d = 3 - 2*y;

        while (x <= y) {

            if (fill) {

                // Верхний и нижний сектор
                this.line(xc - x, yc + y, xc + x, yc + y, color);
                this.line(xc - x, yc - y, xc + x, yc - y, color);

                // Левый и правый сектор
                this.line(xc - y, yc + x, xc + y, yc + x, color);
                this.line(xc - y, yc - x, xc + y, yc - x, color);
            }
            else {

                // Верхний и нижний сектор
                this.pset(xc - x, yc + y, color);
                this.pset(xc + x, yc + y, color);
                this.pset(xc - x, yc - y, color);
                this.pset(xc + x, yc - y, color);

                // Левый и правый сектор
                this.pset(xc - y, yc + x, color);
                this.pset(xc + y, yc + x, color);
                this.pset(xc - y, yc - x, color);
                this.pset(xc + y, yc - x, color);
            }

            d += (4*x + 6);
            if (d >= 0) {
                d += 4*(1 - y);
                y--;
            }

            x++;
        }
    }

    // РАБОТА С ТЕКСТОМ
    // =================================================================

    // Выбор шрифта 8 или 16
    font(font_height = 16) {

        this._font_height = font_height;
        return this;
    }

    // Локация по (X,Y) в текстовом режиме
    locate(x, y) {

        this._locate_x = Math.trunc(x * 8);
        this._locate_y = Math.trunc(y * this._font_height);
        return this;
    }

    // Установка цвета
    color(fore = 0xFFFFFF, back = 0) {

        this._fore = fore;
        this._back = back;
        return this;
    }

    // Печать строки
    print(str) {

        str = str.toString();
        for (let i = 0; i < str.length; i++) {
            this._print_char(str.charCodeAt(i));
        }
    }

    // Печать символа; при указании phantom выводит результат в return
    print_font(x, y, ch, phantom = 0) {

        let fore = this._fore;
        let back = this._back;
        let font = this._font_height === 8 ? this._font8 : this._font16;
        let fh   = this._font_height;
        let out  = [];

        // Если это не число, а символ
        if (typeof ch === "string") {
            ch = ch.charCodeAt(0);
        }

        if (ch >= 1088)
            ch = (ch - 1088) + (this._font_height === 16 ? 0xE0 : 0xF0);
        else if (ch >= 1040)
            ch = (ch - 1040) + (this._font_height === 16 ? 0x80 : 0xC0) ;

        if (ch >= 0x100)
            return;

        for (let i = 0; i < fh; i++) {
            let mask = parseInt(font[ch * fh + i]);
            for (let j = 0; j < 8; j++) {
                let cl = mask & (0x80 >> j) ? fore : back;
                if (phantom) {
                    out[n++] = cl;
                } else if (cl !== null) {
                    this.pset(x + j, y + i, cl);
                }
            }
        }

        return out;
    }

    // Старт: расчет в миллисекундах времени работы от `profiling` до `measure`
    measure(type = 0) {

        if (type) {

            let n = (new Date()).getTime();
            let t = n - this._prof_mc;
            this._prof_mc = n;
            console.log("Measure (" + type + "): " + t + " mc.");
            return t;

        } else {

            this._prof_mc = (new Date()).getTime();
            return 0;
        }
    }

    // РУТИНЫ
    // =================================================================

    // Создается палитра из представленных интервалов
    // [ [index,r,b,g], ... , [index,r,b,g] ]
    make_palette(list) {

        let size = list.length;
        let pal  = {};

        for (let id = 0; id < size-1; id++) {

            let from  = list[id];
            let to    = list[id+1];
            let range = list[id+1][0] - list[id][0];

            // Выстроить палитру
            for (let j = from[0]; j <= to[0]; j++) {

                let f = (j - from[0]) / range;
                let r = Math.floor(f*(to[1] - from[1]) + from[1]);
                let g = Math.floor(f*(to[2] - from[2]) + from[2]);
                let b = Math.floor(f*(to[3] - from[3]) + from[3]);

                // Ограничения
                if (r < 0) r = 0; else if (r > 255) r = 255;
                if (g < 0) g = 0; else if (g > 255) g = 255;
                if (b < 0) b = 0; else if (b > 255) b = 255;

                pal[j] = r*65536 + g*256 + b;
            }
        }

        return pal;
    }

    // Создать cols x rows массив
    dim2(cols, rows, value = 0) {

        let x = new Array(cols);
        for (let i = 0; i < x.length; i++) {
            x[i] = (new Array(rows)).fill(value);
        }
        return x;
    }

    // Случайное целое число от f до t
    irand(f, t = null) {

        if (t === null) {
            t = f - 1;
            f = 0;
        }

        return Math.trunc(Math.random() * (t - f) + f);
    }

    // Псевдослучайное
    trand() {

        let [S0, S1] = this.srand;
        let [N0, N1] = [S0 >> 1, (S1 >> 1) | ((S0 & 1) ? 0x8000 : 0)];
        let H = ((S0 >> 15) ^ (S0 >> 14) ^ (S0 >> 13) ^ (S0 >> 11) ^ (S0 >> 9) ^ S1) & 1;
        this.srand = [N0 | (H ? 0x8000 : 0), N1];
        return N1;
    }

    // Таймаут
    set_timeout(t) {
        this._frame_time = t;
    }

    // Остановка выполнения стандартной рутины
    prevent() {

        this._event.preventDefault();
        this._event.stopPropagation();
    }

    // РАБОТА С АУДИО
    // =================================================================

    // https://developer.mozilla.org/en-US/docs/Web/API/BaseAudioContext/createBuffer
    // Создание буфера (len - в секундах)
    audio_init(len = 1, loop = true) {

        let ctx = new (window.AudioContext || window.webkitAudioContext)();
        let buf = ctx.createBuffer(2, len * ctx.sampleRate, ctx.sampleRate)
        let src = ctx.createBufferSource();

        src.buffer  = buf;
        src.loop    = loop;
        src.connect(ctx.destination);
        src.start();

        return {
            buf:     buf,
            src:     src,
            ctx:     ctx,
            rate:    ctx.sampleRate,
            chcnt:   buf.numberOfChannels,
            // Записать в буфер
            write:   function(t, lf, rt = null) {

                if (!this.ctx) return;

                for (let ch = 0; ch < 2; ch++) {

                    const now = this.buf.getChannelData(ch);
                    now[t % this.rate] = lf;
                    now[t % this.rate] = rt === null ? lf : rt;
                }
            },
            // Текущая проигрываемая позиция
            pos: function() {

                if (this.ctx) {

                    let ct = this.ctx.currentTime;
                    ct = Math.trunc((ct - Math.trunc(ct)) * this.rate);
                    return ct;

                } else {
                    return 0;
                }

            },
        }
    }
}
